"""
Implements a means of using the Bootstrap html and css toolkit to make the report.
"""

from .html import HTMLElement
from itertools import cycle
import markdown
import tabulate

from .html import *
            
class Row(HTMLElement):
    def __init__(self, cols=1, size='md', hclass=''):
        """
        Add a new row to the grid.

        Parameters
        ----------
        cols : int, list, tuple, or iterable
           Either the number of columns desired, or an array of the widths. 
           The widths are specified in twelfths of the full width, so for example 
           `cols = [4,4,4]` makes three columns each one-third of a page wide. 

        size : {'xs', 'sm', 'md', 'lg'}
           The size of the column, according to the Bootstrap responsive design. Defaults to 'md'.

        hclass : str or iterable
           Adds a class to each column. If an iterable is provided then a seperate class
           can be applied to each column.
        """
        
        self.rowopen = "<div class='row'>"
        self.rowclose = "</div>"
        
        if hasattr(cols, '__getitem__'):
            self.columns = [Column(width, size, hclass) for width, hclass in zip(cols, cycle(hclass))]
        elif isinstance(cols, int):
            width = int(12/cols)
            self.columns = [Column(width, size, hclass) for _ in range(cols)]

    # We want this to degrade nicely so if there's only one column we interface directly with the row
    def __add__(self, item):
        if len(self.columns)==1:
            self.columns[0] + item
        else:
            # Need to throw an error
            pass
        
            
    def __getitem__(self, i):
        return self.columns[i]

    def __setitem__(self, i, item):
        self.columns[i].content = markdown.markdown(item, output_format='xhtml5')
        
    def __repr__(self):
        output = ''
        output += self.rowopen
        for column in self.columns:
            output += repr(column)
        output += self.rowclose
        return output

    
    def __str__(self):
        return self.__repr__()

class Column(HTMLElement):
    def __init__(self, width=12, size="md", hclass=""):
        self.width = width
        self.size = size
        self.hclass = hclass
        self.colopen = "<div class='col-{size}-{width} {hclass}'>"
        self.colclose = "</div>"
        self.content = []

    def __repr__(self):
        output = ''
        output += self.colopen.format(size=self.size, width=self.width, hclass=self.hclass)
        for content in self.content:
                output += str(content)
        output += self.colclose
        return output
    
    def __str__(self):
        return self.__repr__()

# class Collapse(HTMLElement):
#     """
#     Creates a bootstrap collapsable div.
#     """
#     def __init__(self, text
    
class Alert(HTMLElement):
    """
    Creates a Bootstrap alert object in the report.
    """
    def __init__(self, text='',style='info',  hclass=''):
        self.alertopen = "<div class='alert alert-{}' role='alert'>".format(style)
        self.alertclose = "</div>"
        self.content = text

    def __repr__(self):
        output = ''
        output += self.alertopen
        output += self.content
        output += self.alertclose
        return output
        
    def __str__(self):
        return self.__repr__()


class Label(HTMLElement):
    def __init__(self, text="", style="default"):
        """
        Make a new label element.
        """
        self.opening = "<span class='label label-{}'>".format(style)
        self.closing = "</span>"
        self.content = text
    def __repr__(self):
        output = ''
        return self.opening + self.content + self.closing
    def __str__(self):
        return self.__repr__()
    
class Panel(HTMLElement):
    """
    Creates a Bootstrap panel object in the report.
    """
    def __init__(self, title='', footer = '', style='default', hclass=''):
        self.title = title
        self.footer = footer
        self.style = style
        self.hclass = hclass

        self.panopen = "<div class='panel panel-{style} {hclass}'>"
        self.panclose = "</div>"

        self.panbody = "<div class='panel-body'>{}</div>"
        self.panheader = "<div class='panel-heading'>{}</div>"
        self.panfooter = "<div class='panel-footer'>{}</div>"
        
        self.content = []


    def __repr__(self):
        output = ''
        output += self.panopen.format(style=self.style, hclass=self.hclass)
        if self.panheader != '': output += self.panheader.format(self.title)
        contents = ""
        for content in self.content:
                contents += str(content)
        output += self.panbody.format(contents)
        if self.footer != '': output += self.panfooter.format(self.footer)
        output += self.panclose
        return output

    __str__ = __repr__



# -*- coding: utf-8 -*-

#import uuid
import os
from .html import *
from configparser import ConfigParser

from jinja2 import Template, Environment, FileSystemLoader

from pkg_resources import resource_string, resource_stream, resource_filename
default_config = resource_string(__name__, 'otter.conf')



class Otter():
    """
    Otter is a pythonic report writing system designed to produce HTML
    reports for long-running or complex jobs where and iPython
    notebook would be an impractical way of presenting information.
    """
    def __init__(self, filename, config_file=None, **kwargs):
        """
        An Otter report is created by this class.

        Parameters
        ----------
        filename : str
           The path to the location of the report, for example `/home/me/www/report.html`.
        config_file: str
           The location of the config file which should be used to generate the report.
        """

        # Attempt to load in default meta data from a config file
        # At the moment just the current directory, but should
        # extend to look in home directory and environment variable location too

        config = ConfigParser()
        #if not config_file:
        try:
            config.read(default_config)
        except TypeError: # Looks like Python 3
            config.readfp(default_config.decode("utf-8"))
        if config_file:
            config.read_string(config_file)
        self.meta = {}

        if config.has_section("meta"):
            for option in config['meta']:
                self.meta[option] = config.get('meta', option)
            for option in kwargs.items():
                self.meta[option[0]] = option[1]
            
        try:
            theme = config.get("theme", "location")
        except:
            print("Cannot find theme in the config file. Using the default theme.")
            try:
                theme = resource_filename(__name__, "themes/default/")
            except:
                print("No theme files found.")
            
        self.env = Environment(loader=FileSystemLoader(theme))
        
        self.reportfolder = filename+"_files"
        self.foldername = os.path.basename(filename)+"_files/"
        if not os.path.exists(self.reportfolder):
            os.makedirs(self.reportfolder)
        #self.reportfile= open(filename,"w")
        self.reportfile = filename
        self.meta.update(kwargs)
        self.items = []

    # Make an otter report work as a context manager
    def __enter__(self):
        """
        Execute this code when the context manager is created.

        Right now, Otter doesn't actually need anything to be done at the 
        creation of a context, but that should really change at some point in
        the future.
        """
        pass

    def __exit__(self, type, value, traceback):
        """
        When the context ends, the report needs to be rendered.
        """
        self.show()
        
    def __add__(self, item):
        return self.add(item)
        
    def add(self, item):
        if HTMLElement in type(item).mro():
            self.items.append(item)
        else:
            item_ = HTMLElement()
            item_ + item
            self.items.append(item_)
        return self

    def show(self):
        html = ''
        for item in self.items:
            html += repr(item)
        output_html = self.env.get_template('body.html').render(meta=self.meta, body=html)
        self._write(output_html)
        
    def _write(self, text):
        with open(self.reportfile, "w") as f:
            f.write(text)

    def _mkdir_recursive(self, path):
        """
        Recursively create the directories required for the report.
        Based off code from http://stackoverflow.com/questions/6004073/how-can-i-create-directories-recursively
        by `Mars'.
        """
        sub_path = os.path.dirname(path)
        if not os.path.exists(sub_path):
            self._mkdir_recursive(sub_path)
        if not os.path.exists(path):
            os.mkdir(path)






# -*- coding: utf-8 -*-

__author__ = 'Daniel Williams'
__email__ = 'd.williams.2@research.gla.ac.uk'

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions


from .otter import *




import matplotlib, numpy
from . import plot
import markdown
import tabulate

md_extensions = [
    'markdown.extensions.tables',
    'markdown.extensions.extra'
    ]


class HTMLElement(object):
    tag = None
    childtag = None
    def __init__(self, content=None, **kwargs):
        self.content = []
        self.meta = kwargs
        if content:
            self.__add__(content)

    def __repr__(self):
        output = ""
        attributes = ""
        for attr, val in self.meta.items():
            if attr=="cl": attr="class"
            attributes += " {}={}".format(attr, val)
        if self.tag: output += "<{} {}>".format(self.tag, attributes)
        for item in self.content:
            if self.childtag:
                output += "<{0}>{1}</{0}>".format(self.childtag, str(item))
            else:
                output += str(item)
        if self.tag: output += "</{}>".format(self.tag)
        return output

    def __str__(self):
        return self.__repr__()
    
    def __iadd__(self, item):
        self.__add__(item)
        return self.content
        
    def __add__(self, item):
        #if isinstance(item, list) and isinstance(item[0], list):
        #    self.content.append( )

        if type(item) in handlers.keys():
            self.content.append(handlers[type(item)](item))
        else:
            self.content.append(item)


class OrderedList(HTMLElement):
    tag = "ol"
    childtag = "li"

    def __add__(self, items):
        for item in items:
            if type(item) in handlers.keys():
                self.content.append(handlers[type(item)](item))
            else:
                self.content.append(item)


      
class Table(HTMLElement):
    tag = "table"

class Row(HTMLElement):
    tag = "tr"
    childtag = "td"
    
    def __add__(self, items):
        for item in items:
            if type(item) in handlers.keys():
                self.content.append(handlers[type(item)](item))
            else:
                self.content.append(item)

def dict_to_table(dictionary):
    table = Table(cl="table")
    for key, val in dictionary.items():
        table + Row([key, val])
    return table

handlers = {
    str: lambda x: markdown.markdown(str(x), output_format='xhtml5', extensions=md_extensions),
    matplotlib.figure.Figure: plot.Figure,
    list: OrderedList,
    dict: dict_to_table,
    numpy.ndarray: lambda x: tabulate.tabulate(x, tablefmt=MyHTMLFormat)
}

from functools import partial
def my_html_row_with_attrs(celltag, cell_values, colwidths, colaligns):
    alignment = { "left":    '',
                  "right":   ' style="text-align: right;"',
                  "center":  ' style="text-align: center;"',
                  "decimal": ' style="text-align: right;"' }
    values_with_attrs =\
        ["<{0}{1} class=\"my-cell\">{2}</{0}>"
            .format(celltag, alignment.get(a, ''), c)
         for c, a in zip(cell_values, colaligns)]
    return "<tr class=\"my-row\">" + \
            "".join(values_with_attrs).rstrip() + \
            "</tr>"
MyHTMLFormat = tabulate.TableFormat(
        lineabove=tabulate.Line("<table class=\"table\">", "", "", ""),
        linebelowheader=None,
        linebetweenrows=None,
        linebelow=tabulate.Line("</table>", "", "", ""),
        headerrow=partial(my_html_row_with_attrs, "th"),
        datarow=partial(my_html_row_with_attrs, "td"),
        padding=0, with_header_hide=None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    'tabulate',
    'jinja2',
    'markdown',
    'pyyaml',
    'matplotlib',
    'configparser'
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='otter-report',
    description="The simple HTML report generator for Python jobs.",
    long_description=readme + '\n\n' + history,
    author="Daniel Williams",
    author_email='daniel.williams@gla.ac.uk',
    url='https://github.com/transientlunatic/otter',
    packages=[
        'otter',
    ],
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    package_dir={
        'otter': 'otter'
    },
    include_package_data=True,
    install_requires=requirements,
    python_requires= ">=3.3",
    license="ISCL",
    zip_safe=False,
    keywords='otter',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



"""
Implements an interface with the Matplotlib plotting library in order to add plots to a report.
"""

import uuid
import matplotlib
matplotlib.use('Agg')
import base64

from io import BytesIO

import matplotlib.pyplot as plt

class Figure():
    def __init__(self, figure, filename=None, dpi=300):

        self.image = BytesIO()

        if filename==None:
            figure.savefig(self.image, dpi=dpi)
                
        self.bitstring = base64.encodestring(self.image.getvalue())
    
    def __repr__(self):
        html_str= r"""<img src="data:image/png;base64,{}" style="max-width: 100%;" class="img-responsive"/>""".format(self.bitstring.decode("utf-8"))
        return html_str
